#!/bin/perl
#################################################################################
# Created       : Martin Foster
# Modified      : 08/10/2002
#################################################################################
#
# Configure Puppet - Script part of Webchat designed to configure Puppets
# Copyright (C) 2000-2002  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204, 817 5TH ST NE
#                 Calgary, Alberta
#                 T2E 3W9
#
#################################################################################

use CGI qw(:standard);						# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error Logs
use strict;							# Strict variable enforcement

use Image::Size;						# Image sizer
use Digest::MD5 qw(md5_hex);					# MD5 digest cryptographic algorithm
use LWP::UserAgent;						# WWW Package

use Ethereal::Database;						# Database handler
use Ethereal::Filter;						# Filter system
use Ethereal::Login;						# Login functionality
use Ethereal::Option;						# Options handling
use Ethereal::Param;						# Parameter control
use Ethereal::Table;						# Table control


#################################################################################
# Data Members
#################################################################################
my $cgi;							# Common gateway interface handle
my $database;							# Database handle
my $login;							# Login handle
my $param;							# Parameter handle

my %config;							# Configuration tag

#################################################################################
# Program Area
#################################################################################

	# Initial handles
	$cgi      = new CGI;
	$database = new Ethereal::Database();
	$param    = new Ethereal::Param();

	# Connect and fetch
	$database->Connect();
	$param->GetParam($cgi);

	# Fetch hash
	$database->GetHashConfiguration(\%config);


	# Document header
	print $cgi->header();
	$database->DocumentGetHeader();

	# Login handle
	$login = new Ethereal::Login($database, $cgi, $param);

	# Authenthication
	if ($login->GetVerificationNormal())
	{
		# Print title
		print "$config{'TagTitle'}\n<BR>\n";
	
		# For additions and removal
		unless  ((defined($param->{$config{'TxtSubmitView'}})) 
                     || (defined($param->{$config{'TxtSubmitButton'}})))
		{ InitialDisplay($database, $cgi, $param, \%config); }

		# For puppet changes and viewing
		else
		{ PuppetControl($database, $cgi, $param, \%config); }
	}

	# Document footer	
	$database->DocumentGetFooter();


#################################################################################
# Sub-Routines
#################################################################################

#####################
# Image Legal
#
# Determines if an image is actually legal based on a regular expression search
# through a list of known banned images.

sub ImageLegal
{
	####################
	# Data Members

	my $database = shift;					# Database handle	
	my $link     = shift;					# Link to image

	my $ban;						# Specific ban
	my @bans;						# List of bans


	####################
	# Program Area

	# Get list of banned images
	$database->GetListImageBan(\@bans);


	# Cycle through
	foreach $ban (@bans)
	{
		# Verify against link
		if ($link =~ /^$ban/si)
		{
			# Return illegal
			return 0;
		}		
	}


	# Return legal
	return 1;
}

#####################
# Image Size
#
# Downloads a picture off the net and determins it's size as to be able place 
# the defined sizes on the database

sub ImageSize
{
        ####################
        # Data Members

	my $database = shift;					# Database handle	
        my $link     = shift;					# URL to picture

        my $name;						# Name of file
	my $ua;							# LWP user agent
	my $request;						# Connection request
	my $response;						# Connection response
	my $size;						# Image file size

	my @sizes;						# Size returned
	my %system;						# System hash


        ####################
        # Program Area

	# Link to Hash
	$database->GetHashSystem(\%system);

        # Initialize handles
	$ua = new LWP::UserAgent();

        # Generate random name
        $name  = md5_hex(rand());;
	$name  = "$system{'LocTemp'}/$name";

        # Retreive image
 	$request  = HTTP::Request->new('GET', $link);
	$response = $ua->request($request, $name);


	# Determine image demensions
	@sizes = imgsize("$name");

	# Dertermine file filse
	$size = -s $name;
	$size = int($size/1024);

	# Remove file
	unlink("$name");

	# Add to array
	unshift(@sizes, $size);


	# Return values
	return @sizes;
}


#####################
# InitialDisplay
#
# Handles the general output of the initial display screen.  Which will allow one
# to change password, E-Mail addresses and/or passwords.

sub InitialDisplay						# Database, CGI, Param
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle
	my $config   = shift;					# Config hash

	my $data     = $database->{'HANDLE'};			# Database connection handle
	my $filter   = new Ethereal::Filter();			# Filter handle
	my $table    = new Ethereal::Table($cgi);		# Table handle

	my @puppets;						# Array of puppets

	my $list;						# List of puppets
	my $buttonview;						# Button view
	my $buttonrem;						# Button remove
	my $buttons;						# Buttons used
	my $conf;						# Confirmation

	my $choice;						# Choice of the matter
	my $message;						# Display message


	#####################
	# Program area	
	
	# Addition operation
	if ( defined($param->{$config->{'TxtSubmitAddition'}}) ) 
	{
		# Strip HTML
		$param->{'PUPADD'} = $filter->StripHTML($param->{'PUPADD'});

		# Additional filtering
		$param->{'PUPADD'} =~ s/~//gs;


		# Retreive information
		($choice) = $database->DataGet("SELECT PuppetName 
			FROM  Puppet 
			WHERE PuppetName=?
			  AND (PuppeteerLogin=?
			   OR PuppetPool=?)", 

		 $param->{'PUPADD'},
		 $param->{'USER'},
		 'system');


		# Verification
		if ((length($param->{'PUPADD'}) > 2)
		 && ($param->{'PUPADD'} !~ /^\s/)
		 && (!defined($choice)))
		{
			# Insertion upon acceptance
			$data->do("INSERT INTO Puppet 
				(PuppetName, 
                                 PuppeteerLogin,
				 PuppetTextSize)
				VALUES(?,?,?)", {},
			 $param->{'PUPADD'}, $param->{'USER'}, 3);

			# Display status
			$message = $config->{'MsgCreate'};
		}
		else
		{ $message = $config->{'MsgExists'}; }
	}

	# Removal operation
	if ((defined($param->{$config->{'TxtSubmitDelete'}}))
	 && (defined($param->{'CONFIRM'})))
	{
		# Retreive information
		($choice) = $database->DataGet('SELECT PuppetName
			FROM Puppet 
			WHERE PuppetName=? 
			  AND PuppeteerLogin=?', 

		 $param->{'PUPPET'}, 
		 $param->{'USER'});


		# Verification
		if (defined($choice))
		{
			# Removal of secondary tables
			$data->do("DELETE FROM PuppetInvitation WHERE PuppetName=?", {}, $param->{'PUPPET'} ); 
			$data->do("DELETE FROM PuppetIgnore WHERE PuppetName=?",     {}, $param->{'PUPPET'} ); 

			# Removal of primary tables
			$data->do("DELETE FROM Puppet 
				WHERE PuppetName=?
				  AND PuppeteerLogin=?", {}, 
			 $param->{'PUPPET'},
			 $param->{'USER'});


			# Message of success
			$message = $config->{'MsgDeletePass'}; 
		}		
		else
		{ $message = $config->{'MsgDeleteFail'}; }
	}


	# Cleanup
	$param->Cleanup(
		'CONFIRM',
		$config->{'TxtSubmitAddition'},
		$config->{'TxtSubmitDelete'});


	# Display of values
	# Retreive a list of puppets
	$database->GetList(\@puppets, "SELECT PuppetName 
		FROM Puppet 
		WHERE PuppeteerLogin=? 
		  AND PuppetActive IS NULL
		ORDER BY PuppetName", 

	$param->{'USER'});


	# Prepare widgets
	$list       = $cgi->scrolling_list('PUPPET', \@puppets, $puppets[0], 10);
	$buttonview = $cgi->submit($config->{'TxtSubmitView'});
	$buttonrem  = $cgi->submit($config->{'TxtSubmitDelete'});
	$conf       = $cgi->checkbox(-name=>'CONFIRM', -label=>'');
	$buttons    = "$buttonview<BR><BR>$buttonrem &nbsp; $conf";

	# Selection/removal
	# Start form
	print $cgi->start_form();
	print $param->EmbedNormal($cgi, %{$param}), "\n";


	# Table top
	$table->MakeTop();

	# Status message
	if (defined($message)) 
	{ 
		$table->MakeSingle($message); 
		$table->MakeBlank(); 
	}
	
	# Select puppet
	$table->MakeSingle($config->{'TagPupDelSelect'});
	$table->MakeValid($list, $buttons);
	$table->MakeBlank();

	# Select kit
	$table->MakeSingle($config->{'TagPupAddition'});
	$table->MakeValid($cgi->textfield('PUPADD', '', 20, 30), $cgi->submit($config->{'TxtSubmitAddition'}));
	$table->MakeBottom();

	# End form
	print $cgi->end_form();
}


#####################
# PuppetControl
#
# Sub-routine that will handle the change of information for a puppet.  This
# is assuming that the submit button was pressed.

sub PuppetControl
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle
	my $config   = shift;					# Config hash

	my $data     = $database->{'HANDLE'};			# Database connection handle
	my $option   = new Ethereal::Option();			# Option handle
	my $table    = new Ethereal::Table($cgi);		# Table handle

	my $statement;						# Query handle

	my $choice   = 0;					# Choice of the matter

	my $admin;						# Administrative access
	my $mod;						# Moderator access
	my $super;						# Supervisory access
	my $ver;						# Verified user

	my $info;						# Puppet information
	my $gummies;						# Gummies html
	my $gummy;						# Default entry

	my $inline;						# Parameters
	my $url      = $cgi->url();				# Self reference

	my @gummies;						# List of gummies


	#####################
	# Program area

	# Change of information and verify for appripriate button press
	if ( defined($param->{$config->{'TxtSubmitButton'}}) )
	{ 
		PuppetControlWrite($database, $param, $config); 
	}


	# Data Retreival
	# Determin privileges
	($super, $admin, $mod, $ver) = $database->DataGetPrivs($param->{'USER'});


	# Back button
	# Generate Inline
	$inline = $param->EmbedInline($cgi, 'USER', $param->{'USER'}, 'CRYPT', $param->{'CRYPT'});	


	# Gather information
	# Prepare and execute
	$statement = $data->prepare("SELECT PuppetName,
			 PuppetActive,
			 PuppetBio,
			 PuppetDefault,
			 PuppetDisplay,
			 PuppetGummy,
			 PuppetPic,
			 PuppetPicHeight,
			 PuppetPicSize,
			 PuppetPicWidth,
			 PuppetPool,
			 PuppetTag,
			 PuppetTextColour,
			 PuppetTextSize,
			 PuppetTitle
			FROM  Puppet
			WHERE PuppetName=? 
			  AND PuppeteerLogin=?");
	$statement->execute($param->{'PUPPET'}, $param->{'USER'});

	# Retreive into hash
	$info = $statement->fetchrow_hashref();


	# Gummy list
	# Retreive icons general or canon
	if ($info->{'PuppetPool'} eq 'system')
	{
		# Default and general
		GetGummyDefault($database, \@gummies);
		GetGummyGeneral($database, \@gummies);
	}
	else
	{
		# Simply canon handles
		GetGummyCanon($database, \@gummies);
	}

	# Administrative, Supservisory and Moderator gummies
	GetGummyContrib($database, \@gummies) if ($ver   eq 'yes');
	GetGummyAdmin($database, \@gummies)   if ($admin eq 'yes');
	GetGummySuper($database, \@gummies)   if ($super eq 'yes');
	GetGummyMod($database, \@gummies)     if ($mod   eq 'yes');

	# Reverse lookup gummy
	($gummy)  = $database->DataGetGummyName($info->{'PuppetGummy'});

	# Determin default and format
	$gummies  = $cgi->popup_menu('GUMMY', \@gummies, $gummy);


	# Add hint for users
	$info->{'PuppetPic'} = (defined($info->{'PuppetPic'})) ? $info->{'PuppetPic'} : 'http://';
	$info->{'PuppetBio'} = (defined($info->{'PuppetBio'})) ? $info->{'PuppetBio'} : 'http://';


	# Display of information
	# Star form
	print $cgi->start_form();
	print $param->EmbedNormal($cgi, %{$param}, 'PUPPET', $param->{'PUPPET'}, 'POOL', $info->{'PuppetPool'}), "\n";

	# Table top
	$table->MakeTop();

	# Back button
	$table->MakeSingle("<A HREF=\"$url$inline\">$config->{TagBack}</A>");
	$table->MakeBlank();

	# Most clueful information
	$table->MakeValid($config->{'TagPupName'}, "<B>$info->{'PuppetName'}</B>");
	$table->MakeBlank();

	# Image information
	$table->MakeValid($config->{'TagPupImageLink'}, $cgi->textfield('PUPIMG', $info->{'PuppetPic'}, 30, 200));
	$table->MakeValid($config->{'TagPupImgWidth'},  "<I>$info->{'PuppetPicWidth'}</I>");
	$table->MakeValid($config->{'TagPupImgHeight'}, "<I>$info->{'PuppetPicHeight'}</I>");
	$table->MakeValid($config->{'TagPupImgSize'},   "<I>$info->{'PuppetPicSize'}</I>");
	$table->MakeBlank();

	# Bio Url
	$table->MakeValid($config->{'TagPupBio'}, $cgi->textfield('PUPBIO', $info->{'PuppetBio'}, 30, 200));
	$table->MakeValid($config->{'TagPupGummy'}, $gummies);
	$table->MakeBlank();

	# Checkboxes
	# Active Puppets
	unless (defined($info->{'PuppetActive'}))
	{ 
		# Active
		$table->MakeValid(
			$config->{'TagPupActive'}, 
			$cgi->checkbox(-name=>'PUPACT', -checked=>'on', -label=>'')); 
	}
	else
	{ 
		# Inactive
		$table->MakeValid(
			$config->{'TagPupActive'}, 
			$cgi->checkbox(-name=>'PUPACT', -label=>'')); 
	}

	# Default Puppets
	if ($info->{'PuppetDefault'} eq 'yes')
	{ 	
		# Active
		$table->MakeValid(
			$config->{'TagPupDefault'}, 
			$cgi->checkbox(-name=>'PUPDEF', -checked=>'on', -label=>'')); 
	}
	else
	{ 
		# Inactive
		$table->MakeValid(
			$config->{'TagPupDefault'}, 
			$cgi->checkbox(-name=>'PUPDEF', -label=>'')); 
	}


	# Display Puppeteer information
	if ($info->{'PuppetDisplay'} eq 'yes')
	{ 	
		# Active
		$table->MakeValid(
			$config->{'TagPupDisplay'}, 
			$cgi->checkbox(-name=>'PUPDIS', -checked=>'on', -label=>'')); 
	}
	else
	{ 
		# Inactive
		$table->MakeValid(
			$config->{'TagPupDisplay'}, 
			$cgi->checkbox(-name=>'PUPDIS', -label=>'')); 
	}

	# Spacer
	$table->MakeBlank();


	# More complexe title
	$table->MakeSingle($config->{'TagPupTitle'});
	$table->MakeSingle($cgi->textarea(-name=>'PUPTIT',-rows=>3,-columns=>65,-wrap=>'SOFT',-default=>$info->{'PuppetTitle'}));
	$table->MakeBlank();

	# More complexe tagline
	$table->MakeSingle($config->{'TagPupTagline'});
	$table->MakeSingle($cgi->textarea(-name=>'PUPTAG',-rows=>3,-columns=>65,-wrap=>'SOFT',-default=>$info->{'PuppetTag'}));
	$table->MakeBlank();


	# Finish table
	$table->MakeValid($cgi->submit($config->{'TxtSubmitButton'}), $cgi->reset($config->{'TxtResetButton'}));
	$table->MakeBottom();


	# End of form
	print $cgi->end_form();

	# Finish statement
	$statement->finish();
}


#####################
# PuppetControlWrite
#
# Sub-routine that will handle the change of information for a puppet.  This
# is used as a sub process to handle database interaction.

sub PuppetControlWrite
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $param    = shift;					# Parameter handle
	my $config   = shift;					# Config hash

	my $data     = $database->{'HANDLE'};			# Database connection handle

	my $filter   = new Ethereal::Filter();			# Filter handle
	my $option   = new Ethereal::Option();			# Option handle

	my $legal;						# Image legal
	my $height;						# Image height
	my $size;						# Image size
	my $width;						# Image width

	my $cbio;						# Bio
	my $cgum;						# Gummy
	my $ctag;						# Cleaned tag
	my $ctit;						# Cleaned title

	my $active;						# Active flag
	my $default;						# Default toggle
	my $display;						# Display of information
	my $file;						# Filename
	my $name;						# Name of gummy

	my $lang;						# Language filter
	my @lang;						# Array of banned words


	#####################
	# Program area

	# Determine active state
	($active) = $database->DataGetPuppetActive($param->{'PUPPET'}, $param->{'USER'});


	# Image handling
	# If valid URL
	if ((length($param->{'PUPIMG'}) > 10)
	 && ($param->{'PUPIMG'} !~ /^http:\/\/\d{1,3}\.\d{1,3}\.\d{1,3}/i)) 
	{
		# If legal
		if (ImageLegal($database, $param->{'PUPIMG'}))
		{	
			# Capture and determin size
			($size, $width, $height, $_) = ImageSize($database, $param->{'PUPIMG'}); 

			# Make sure image is valid
			$param->{'PUPIMG'} = defined($width) ? $param->{'PUPIMG'} : undef;

	
			# Update database with new information
			$data->do("UPDATE Puppet
				SET PuppetPic=?, 
				    PuppetPicWidth=?, 
				    PuppetPicHeight=?,
				    PuppetPicSize=?
				WHERE PuppetName=? 
				  AND PuppeteerLogin=?", {}, 
			 $param->{'PUPIMG'}, 
			 $width, 
			 $height, 
			 $size,
			 $param->{'PUPPET'}, 
			 $param->{'USER'}); 
		}
	}
	else
	{
		# Update database with nulls
		$data->do("UPDATE Puppet 
			SET PuppetPic=NULL, 
		            PuppetPicWidth=NULL, 
			    PuppetPicHeight=NULL,
			    PuppetPicSize=NULL
			WHERE PuppetName=? 
			  AND PuppeteerLogin=?", {}, 

		 $param->{'PUPPET'}, 
		 $param->{'USER'}); 
	}


	# Assign default
	if (defined($param->{'PUPDEF'}))
	{
		# Database updates
		# Pull any stray tags
		$data->do("UPDATE Puppet
			SET PuppetDefault='no'
			WHERE PuppeteerLogin=?
			  AND PuppetDefault=?", {},

		 $param->{'USER'},
		 'yes');

		# Assign proper default
		$data->do("UPDATE Puppet
			SET PuppetDefault='yes'
			WHERE PuppetName=?
			  AND PuppeteerLogin=?
			  AND PuppetPool=?", {},

		 $param->{'PUPPET'},
		 $param->{'USER'},
		 'system');
	}


	# Active puppets
	# Activate
	if ((defined($param->{'PUPACT'})) 
	 && (defined($active)))
	{
		# Remove timestamp
		$data->do("UPDATE Puppet
			SET PuppetActive=NULL
			WHERE PuppetName=?
			  AND PuppeteerLogin=?", {},

		 $param->{'PUPPET'},
		 $param->{'USER'});
	}

	# Deactivate
	elsif ((!defined($param->{'PUPACT'}))
	 && (!defined($active)))
	{
		# Remove timestamp
		$data->do("UPDATE Puppet
			SET PuppetActive=?
			WHERE PuppetName=?
			  AND PuppeteerLogin=?", {},

		 time,
		 $param->{'PUPPET'},
		 $param->{'USER'});
	}


	# Data handling
	# Truncate and clean output
	$ctag = substr($param->{'PUPTAG'}, 0, 1024);
	$ctit = substr($param->{'PUPTIT'}, 0, 1024);
	$cbio = $param->{'PUPBIO'};

	# Pull value
	$display = $param->{'PUPDIS'};


	# Gummy handling
	($cgum) = $database->DataGetGummyPath($param->{'GUMMY'});	


	# Tag handling
	($ctag) = $filter->SafeHTML($ctag);
	($ctit) = $filter->SafeHTML($ctit);

	# Language filter
	# Retreive all words
	$database->GetList(\@lang, "SELECT RestrictionValue
		FROM  Restriction
		WHERE RestrictionType='word'
		GROUP BY RestrictionValue");

	# Language filtering
	while ($lang = shift(@lang))
	{
		# Run a global replace
		$ctag =~ s/$lang/****/gsi;
		$ctit =~ s/$lang/****/gsi;
	}


	# Database update
	# Tagline, Title and Bio link
 	$ctag = (length($ctag) > 3)  ? $ctag : undef;
 	$ctit = (length($ctit) > 3)  ? $ctit : undef;
 	$cbio = ((length($cbio) > 12) && ($cbio =~ /^http:\/\//is)) ? $cbio : undef; 

	# Information display
	$display = (defined($display)) ? 'yes' : 'no';


	# Update fields
	$data->do("UPDATE Puppet 
		SET PuppetTag=?, 
		    PuppetTitle=?,
		    PuppetBio=?,
		    PuppetGummy=?,
		    PuppetDisplay=? 
		WHERE PuppetName=? 
		  AND PuppeteerLogin=?", {}, 
	 $ctag, 
	 $ctit, 
	 $cbio,
	 $cgum, 
	 $display,
	 $param->{'PUPPET'}, 
	 $param->{'USER'});


	# Cleanup
	$param->Cleanup('POOL', 'PUPACT', 'PUPDEF', 'PUPDIS');
}

#####################
# Puppet Gummy
#
# This function is a variant of the GetList function used to pull
# fields from the database.  However, since this one will not
# clean up the array which is generally desired. 

sub PuppetGummy
{
	#####################
	# Data members
	my $database = shift;					# Database handle

	my $data     = $database->{'HANDLE'};			# Direct handle
	my $statement;						# Database statement

	my $array    = shift;					# Reference to array
        my $class    = shift;					# Identification class                             
	my $realm    = shift;					# Realm name


	#####################
	# Program Area

	# Define realm
	$realm = (defined($realm)) ? $realm : 'system';


	# Prepare and execute
	$statement = $data->prepare("SELECT GummyName 
		FROM Gummy 
		WHERE GummyClass=?
		  AND RealmName=?
		ORDER BY GummyIdent, 
			 GummyName");
	$statement->execute($class, $realm);


	# Retrieve and place in array
	while ($_ = $statement->fetchrow())
	{ push(@{$array}, $_); }

	# End statement
	$statement->finish;
}

# Alias for administrators
sub GetGummyAdmin
{ PuppetGummy(shift, shift, 'admin'); }

# Alias for default gummy
sub GetGummyDefault
{ PuppetGummy(shift, shift, 'default'); }

# Alias for canon handles
sub GetGummyCanon
{ PuppetGummy(shift, shift, 'canon'); }

# Alias for contributor gummies
sub GetGummyContrib
{ PuppetGummy(shift, shift, 'contributor'); }

# Alias for general
sub GetGummyGeneral
{ PuppetGummy(shift, shift, 'general'); }

# Alias for moderators
sub GetGummyMod
{ PuppetGummy(shift, shift, 'mod'); }

# Alias for supervisors
sub GetGummySuper
{ PuppetGummy(shift, shift, 'super'); }
