#!/bin/perl
################################################################################
# Created       : Martin Foster
# Modified      : 15/10/2002
################################################################################
#
# Portal - Script part of Webchat designed to authenthicate and and lead to realms
# Copyright (C) 2000-2002  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204 - 817, 5th Street NE
#                 Calgary, Alberta
#                 T2E 3W9
#
################################################################################

use CGI qw(:standard);						# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use Digest::MD5 qw(md5_hex);					# MD5 digest cryptographic algorithm
use strict;							# Strict variable enforcement

use Ethereal::Database;						# Database handler
use Ethereal::Login;						# Login functionality
use Ethereal::Param;						# Parameter control
use Ethereal::Table;						# Table handler
use Ethereal::Weather;						# Weather handler
use Ethereal::Who;						# Who is on


################################################################################
# Data Members
################################################################################
my $cgi;							# Common gateway interface handle
my $database;							# Database handle
my $login;							# Login handle
my $param;							# Parameter hash

my $scriptname;							# Simply the script name
my $scriptpara;							# Scripted parameters

my %portal;							# Portal hash

################################################################################
# Program Area
################################################################################

	# Initial handles
	$cgi      = new CGI;
	$database = new Ethereal::Database();
	$param    = new Ethereal::Param();

	# Connect and fetch
	$database->Connect();
	$param->GetParam($cgi);

	# Link with hash
	$database->GetHashPortal(\%portal);


	# Pull script name
	$scriptname = $cgi->url(-relative=>1);


	# Regular operation
	unless ($scriptname =~ /^static/si)
	{
		# Document header
		print $cgi->header();
		$database->DocumentGetHeader();

		# Login handle
		$login = new Ethereal::Login($database, $cgi, $param);

		# Authenthication
		if ($login->GetVerificationNormal())
		{
			# Quick launch
			if ($scriptname =~ /^go/si)
			{
				# Pull script name
				$scriptname = $cgi->url(-full=>1);
				$scriptpara = $cgi->url(-path_info=>1);

				# Determine need
				# Escape
				$scriptname =~ s/(\W)/\\$1/gs;

				# Truncate
				$scriptpara =~ s/^$scriptname\///;


				# Warp config
				# Need exists
				if (length($scriptpara) > 3)
				{
					# Unescape
					$scriptpara = $cgi->unescape($scriptpara);

					# Parameter assignment
					$param->{'ROOM'} = $scriptpara;
					$param->{'THIRDCHAPTER'} = 'True';
				}
			}

			# If undefined list realms, puppets and links
			unless (defined($param->{'THIRDCHAPTER'}))
			{ 
				SecondChapter($database, $cgi, $param, \%portal); 
			}
	
			# Otherwise open gateway
			elsif (defined($param->{'THIRDCHAPTER'}))
			{ 
				ThirdChapter($database, $cgi, $param, \%portal); 
			}
		}

		# Document footer
		$database->DocumentGetFooter();
	}

	# Static/Non Interactive/SSI handler
	else
	{
		# Pointless but needed
		print "<BR>\n\n";

		# Call for operation
		DescriptionStatic($database, $cgi, \%portal);
	}

################################################################################
# Sub-Routines
################################################################################

#####################
# Description Handler
#
# If javascript is enabled, this function will be called.   Once that is done
# the necessary components will be filled in as to allow for use of the 
# javascript descriptions/

sub DescriptionHandler
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $js       = shift;					# Javascript aware


	#####################
	# Program Area

	# Only if necessary
	if ($js = 'yes')
	{
		#####################
		# Local data members
		my $data     = $database->{'HANDLE'};		# Direct handle

		my $res;					# Query results
		my $statement;					# Query statements

		my $macro ='';					# Generate macro replacement
		my $descr;					# Description


		# Retreive results
		# Description
		$descr = $database->DocumentGetDescription();


		# Realms and descriptions
		$statement = $data->prepare("SELECT
				 RealmName,
				 RealmDescription
				FROM Realm
				WHERE RealmPublic='yes'
				  AND RealmDescription IS NOT NULL");
		$statement->execute();


		# Loop and generate
		while ($res = $statement->fetchrow_hashref())
		{
			# Append to macro
			$macro = "$macro\tcase \"$res->{RealmName}\":\n\t mes='$res->{RealmDescription}'\;\n\t break\;\n";
		}


		# Complete query
		$statement->finish();

		
		# Replace macro
		$descr =~ s/MDESCRCASE/$macro/gs;

		# Display
		print "$descr\n\n";
	}
}


#####################
# Description Static
#
# A variant of the above with purpose of displaying this to mostly
# static pages.  Meaning being used in Server Side Includes (SSI)
# et cetera.

sub DescriptionStatic
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# Common Gateway Interface
	my $portal   = shift;					# Portal handler

	my $data     = $database->{'HANDLE'};			# Direct handle
	my $res;						# Query results
	my $statement;						# Query statements
	my $table;						# Table handler	

	my $url;						# Quick access link
	my $title;						# Displayed title


	#####################
	# Program Area

	# Define instance
	$table = new Ethereal::Table($cgi);


	# Retreive results
	# Realms and descriptions
	$statement = $data->prepare("SELECT
			 RealmName,
			 RealmDescription
			FROM Realm
			WHERE (RealmPublic='yes'
			   OR RealmVerified='yes')
			  AND RealmDescription IS NOT NULL
			ORDER BY RealmName");
	$statement->execute();

	# Table top
	$table->MakeTop("100%");

	# Loop and generate
	while ($res = $statement->fetchrow_hashref())
	{
		# Generate link
		$url = $cgi->escape($res->{'RealmName'});
		$url = $portal->{'LnkQuickLink'} . "/" . $url;

		# Title handler
		$title =  $portal->{'TagSecTitle'};
		$title =~ s/MTITLE/$res->{'RealmName'}/gs;
		$title =~ s/MLINK/$url/gs;


		# Information and spacer
		$table->MakeSingle("$title");
		$table->MakeSingle("<P ALIGN=\"justify\">$res->{RealmDescription}</P>");
		$table->MakeBlank();
	}

	# Table bottom
	$table->MakeBottom();
}

#####################
# Second Chapter
#
# Unusual name given to a part of the script that will be exected once the user
# has been authenthicated.   

sub SecondChapter
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter list
	my $portal   = shift;					# Portal hash

	my $table    = new Ethereal::Table($cgi);		# Table handler 

	my $default;						# Default puppet
	my $inline;						# Inline embeding of data
	my $mail;						# Mail count
	my $news;						# News entries
	my $puppets;						# Formatted list of puppets
	my $realms;						# Formatted list of realms
	my $url;						# Self referencing link

	my $sup;						# Supervisory
	my $adm;						# Administrative
	my $mod;						# Moderators
	my $ver;						# Verified users

	my $js;							# Javascript control
	my $jscode;						# Javascript code

	my @realms;						# List of realms
	my @puppets;						# List of puppets
	my @ver;						# Verified realms


	#####################
	# Program area

	# Retreive URL
	$url = $cgi->url();


	# Javascript Options
	# Retreive Javascript choice
	($js) = $database->DataGetJavascript($param->{'USER'});
	
	# Possibly display Javascript
	if ($js eq 'yes')
	{ 
		# Display header
		$database->DocumentGetJavascript(); 
	}

	# Generate javascript code
	$jscode = ($js eq 'yes') ? "ONCLICK='window.open(this.href,this.target,settings); return false'" : '';


	# Retreive puppet default
	($default) = $database->DataGetDefault($param->{'USER'}); 

	# Retreive numbers
	($mail)    = $database->DataGetPuppeteerMailCount($param->{'USER'});
	($news)    = $database->DataGetWeblogEntries();

	# Determine priviliges
	($sup, $adm, $mod, $ver) = $database->DataGetPrivs($param->{'USER'});


	# Retreive and format values for Realms and Puppets
	$database->GetListPublicRealms(\@realms);	
	$database->GetListPuppetYours(\@puppets, $param->{'USER'});	

	# Verified realms
	if ($ver eq 'yes')
	{
		# Pull list
		$database->GetListVerifiedRealms(\@ver);

		# Append and sort
		@realms = sort(@realms, @ver);
	}



	# Widget handling
	# Puppets
	$puppets = $cgi->scrolling_list('CHAR', \@puppets, $default, 5);

	# Realms
	if ($js eq 'yes')
	{
		# Javascript aware
		$realms = $cgi->scrolling_list(
		  -name=>'ROOM',
		  -values=>\@realms,
		  -default=>$realms[0],
		  -size=>10,
		  -onchange=>'createContent()'
		 );
	}

	else
	{
		# Ignorant of javascript
		# To avoid any error messages
		$realms = $cgi->scrolling_list(
		  -name=>'ROOM',
		  -values=>\@realms,
		  -default=>$realms[0],
		  -size=>10
		 );
	}


	# Display of information
	# Table top
	print "<CENTER>\n<TABLE WIDTH=\"95%\">\n";

	# Row top
	print " <TR VALIGN=\"top\">\n  <TD>\n";

	# Only display if puppets exist
	if (defined($default))
	{
		# Realm Section
		# Start form
		print $cgi->start_form(-NAME=>'PORTAL');
		print $param->EmbedNormal($cgi, %{$param}, 'THIRDCHAPTER', 'true'), "\n";

		# Table top
		$table->MakeTop(400);
		$table->MakeSingle($portal->{'TagSecRealm'});
		$table->MakeBlank();
	
		$table->MakeValid($portal->{'TagSelectPuppet'},  $puppets);
		$table->MakeBlank();
		$table->MakeValid($portal->{'TagSelectRealm'},   $realms);
		$table->MakeBlank();
		$table->MakeValid($portal->{'TagSelectPrivate'}, $cgi->textfield('ALTROOM', '', 20, 30));
		$table->MakeBlank();
	
		$table->MakeValid($cgi->submit($portal->{'TxtSubmitRealms'}), $cgi->reset($portal->{'TxtResetRealms'}));
		$table->MakeBottom();

		# End form
		print $cgi->end_form(), "\n";


		# Spacer
		print "<BR>\n";

		# Show descriptions
		DescriptionHandler($database, $js);

		# Spacer
		print "<BR>\n";
	}

		
	# Row bottom
	print "  </TD>\n";

	# Links
	# Create inline link
	$inline = $param->EmbedInline($cgi, %{$param});


	# Row top
	print "  <TD>\n";


	# Puppeteer Section
	print "$portal->{TagSecPuppeteer}\n";
	print $cgi->ul(
	 $cgi->li("<A HREF=\"$portal->{LnkPuppeteer}$inline\" TARGET=\"PORTAL\" $jscode>$portal->{TagLnkPuppeteer}</A>"),
	 $cgi->li("<A HREF=\"$portal->{LnkPuppet}$inline\"    TARGET=\"PORTAL\" $jscode>$portal->{TagLnkPuppet}</A>"),
	 $cgi->li("<A HREF=\"$portal->{LnkHome}$inline\"      TARGET=\"PORTAL\" $jscode>$portal->{TagLnkHome}</A>"), "\n");


	# Messaging
	print "$portal->{TagSecMessage}\n";
	print $cgi->ul(
	 $cgi->li("<A HREF=\"$portal->{LnkMail}$inline\"      TARGET=\"PORTAL\" $jscode>$portal->{TagLnkMail}</A> ($mail)"),
	 $cgi->li("<A HREF=\"$portal->{LnkBoard}$inline\"     TARGET=\"PORTAL\" $jscode>$portal->{TagLnkBoard}</A>"), 
	 $cgi->li("<A HREF=\"$portal->{LnkSysboard}$inline\"  TARGET=\"PORTAL\" $jscode>$portal->{TagLnkSysboard}</A>"), 
	 $cgi->li("<A HREF=\"$portal->{LnkWeblog}\"           TARGET=\"PORTAL\" $jscode>$portal->{TagLnkWeblog}</A> ($news)"), "\n");

	# Spacer
	print "<BR>\n";


	# Administrative area
	if ($adm eq 'yes')
	{
		# Title
		print "$portal->{TagSecAdmin}\n";
		print $cgi->ul(
		 $cgi->li("<A HREF=\"$portal->{LnkAdmSystem}$inline\" TARGET=\"PORTAL\">$portal->{TagLnkAdmSystem}</A>"),
		 $cgi->li("<A HREF=\"$portal->{LnkAdmWho}$inline\"    TARGET=\"PORTAL\">$portal->{TagLnkAdmWho}</A>"), "\n");

		# Spacer
		print "<BR>\n";
	}


	# Supervisory Area
	if ($sup eq 'yes')
	{
		# Title
		print "$portal->{TagSecSuper}\n";

		# Links
		print $cgi->ul(
		 $cgi->li("<A HREF=\"$portal->{LnkRealm}$inline\"     TARGET=\"PORTAL\" $jscode>$portal->{TagLnkSupervisor}</A>"), "\n"); 

		# Spacer
		print "<BR>\n";
	}


	# Miscellaneous area
	print "$portal->{TagSecMisc}\n";
	print $cgi->ul(
	 $cgi->li("<A HREF=\"$portal->{LnkCookie}$inline\"    TARGET=\"PORTAL\" $jscode>$portal->{TagLnkCookie}</A>"),
	 $cgi->li("<A HREF=\"$portal->{LnkGlobalWho}$inline\" TARGET=\"PORTAL\" $jscode>$portal->{TagLnkGlobalWho}</A>"), "\n");

	# End table
	print "  </TD>\n </TR>\n</TABLE>\n</CENTER>\n";
}


#####################
# Third Chapter
#
# Unusual name given to a part of the script that will be exected once the user  
# in his infinit wisdom has selected a puppet and realm

sub ThirdChapter
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter list
	my $portal   = shift;					# Portal hash

	my $data     = $database->{'HANDLE'};			# Database connection handler
	my $who;						# Who is on handle

	my $access;						# Access control
	my $alias;						# Room alias
	my $room;						# Room name
	my $info;						# Room information
	my $rand;						# Random password
	my $status;						# Room status


	#####################
	# Program area

	# Cleanup
	delete $param->{'THIRDCHAPTER'};


	# Determine need to create a room
	if (defined($param->{'ALTROOM'}) && (length(($param->{'ALTROOM'})) > 2))
	{
		# Existence of password
		if ($param->{'ALTROOM'} =~ /@/)
		{
			# Seperate into password and realm
			($param->{'RPASS'}, $param->{'ALTROOM'}) = split(/@/, $param->{'ALTROOM'});

			# Meets length requirements
			$param->{'ROOM'} = (length(($param->{'ALTROOM'})) > 2) ? $param->{'ALTROOM'} : $param->{'ROOM'};
		}

		# Default
		else
		{	
			# Assign ROOM
			$param->{'ROOM'} = $param->{'ALTROOM'};
		}

		# Retreive query information
		($room, $status, $alias) = $database->DataGetRealmExistence($param->{'ROOM'});


		# Create if not found
		unless (defined($room))
		{
			# Handle realm password
			if ((defined($param->{'RPASS'})) 
			 && (length($param->{'RPASS'}) > 2))
			{
				# Generate password
				$rand  = md5_hex(rand());
				$rand  = substr($rand, 0, 14);

				# Generated encrypted password
				$param->{'RCRYPT'} = crypt($param->{'RPASS'}, $rand);

				# Cleanup
				$param->Cleanup('RPASS');
			}
			else
			{
				# Undefine parameter
				$param->{'RCRYPT'} = undef;
			}

			# Realm creation
			$data->do("INSERT INTO Realm 
				(RealmName,
				 RealmPassword,
				 RealmImageHeight,
				 RealmImageWidth,
				 RealmImageSize,
				 RealmTimestamp)
				VALUES(?,?,0,0,0,?)", {}, 

			 $param->{'ROOM'}, 
			 $param->{'RCRYPT'}, 
			 time); 			
		}
		

		# Realm exists
		else
		{
			# Handle alias
			$room = ($status eq 'alias')  ? $alias : $room;

			# Consistent naming
			$param->{'ROOM'} = $room;
		}
	}

	# Access control
	($access) = $database->DataGetRealmAccess($param->{'ROOM'});

	# Ensure puppet is defined
	if (($access eq 'restricted')
	 or (!defined($param->{'CHAR'})))
	{
		# Retreive default puppet
		($param->{'CHAR'}) = $database->DataGetDefault($param->{'USER'}, $param->{'ROOM'}, $access);
	}

	# Retreive realm information
	($info) = $database->DataGetRealmDescription($param->{'ROOM'});

	
	# who is on handles
	$who = new Ethereal::Who($database, $cgi, $param);
	
	# Wrap with tables
	if (defined($info))
	{	
		# Wrap with tables
		print "<TABLE WIDTH=\"100%\">\n <TR>\n  <TD VALIGN=\"top\">";

		# Retreive realm information
		$who->WhoEntrance();

		# Warn before entering
		$database->DocumentGetCanon() if ($access eq 'restricted');

		# Weather forcast
		WeatherHandler($database, $cgi, $param, $portal);

		# Mid section and info
		print "\n  </TD>\n  <TD VALIGN=\"top\">";
		print "$info\n";

		# Hind quarter
		print "\n  </TD>\n </TR>\n</TABLE>";
	}

	# Simply display
	else
	{
		# Retreive realm information
		$who->WhoEntrance();

		# Warn before entering
		$database->DocumentGetCanon() if ($access eq 'restricted');

		# Weather forcast
		WeatherHandler($database, $cgi, $param, $portal);
	}
}


#####################
# Weather Handler
#
# The weather heandler is realm specific for which is it not listed on the
# module directly.   This will essentially show off the specific weather 
# patterns for the realm if found

sub WeatherHandler
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter list
	my $portal   = shift;					# Portal handle

	my $table    = new Ethereal::Table($cgi);		# Table handle

	my $for;						# Forcast handle
	my $metar;						# Coded weather forcast

	my $cond;						# Weather conditions
	my $sky;						# Sky conditions

	my %cond;						# Specific conditions


	#####################
	# Program area

	# Retreive forcast
	($metar) = $database->DataGetWeather($param->{'ROOM'});
	

	# Only complete if forcast
	if (defined($metar))
	{
		# Initialize handle
		$for = new Ethereal::Weather();

		# Parse
		$for->Parse($metar, \%cond);


		# Forcast handling
		$sky  = lc(join(", ", @{$cond{SKY}}));
		$cond = lc(join(", ", @{$cond{HUMAN}}));		

		# Conditions
		$cond = (length($cond) > 2) ? ucfirst("$cond, $sky") : ucfirst("$sky");
		$cond =~ s/,([^,.]+?)$/ and $1/s;


		# Title
		print "<BR>\n$portal->{'TagForcastTitle'}\n<BR>\n";

		# Display area
		$table->MakeTop("100%");
		$table->MakeSingle("$portal->{TagForcastCond}<BR>$cond");

		# Wind
		if ($cond{WIND_MPH} != 0)
		{ $table->MakeSingle("$portal->{TagForcastWind}<BR>$cond{WIND_DIR_ENG} at $cond{WIND_KPH} KPH - $cond{WIND_MPH} MPH"); }

		# Temperature
		if ($cond{F_TEMP} != 0)
		{ $table->MakeSingle("$portal->{TagForcastTemp}<BR>$cond{C_TEMP}C - $cond{F_TEMP}F" ); }

		# Dew points
		if ($cond{F_DEW} != 0)
		{ $table->MakeSingle("$portal->{TagForcastDew} <BR>$cond{C_DEW}C - $cond{F_DEW}F"); }

		# Visibility
		if ($cond{VIS_MILE} != 0)
		{ $table->MakeSingle("$portal->{TagForcastVis} <BR>$cond{VIS_KILO} Km - $cond{VIS_MILE} Mi"); }

		# Barometer
		if ($cond{ALTI} != 0)
		{ $table->MakeSingle("$portal->{TagForcastBaro}<BR>$cond{ALTI} Inches - $cond{ALTK} Kilopascals"); }

		# Table bottom
		$table->MakeBottom();
	}
}
